/**
  * Map Abstraction
  */

//Constructor
function AbstractMap(element, map) {
	this.element = element; // DOM node
	this.map = map;
}

AbstractMap.prototype.init = function() {
	this.finalize();
}

AbstractMap.prototype.finalize = function() {
	this.onReady();
}

// Should be called when
AbstractMap.prototype.onReady = function() {
	console.log("AbstractMap::onReady()");
}

AbstractMap.prototype.onZoom 		= function() { }
AbstractMap.prototype.onDragStart 	= function() { }
AbstractMap.prototype.onDrag 		= function() { }
AbstractMap.prototype.onDragEnd 	= function() { }
AbstractMap.prototype.onIdle	 	= function() { }
AbstractMap.prototype.onTouchStart 	= function() { }
AbstractMap.prototype.onTouchMove	= function() { }
AbstractMap.prototype.onTouchEnd 	= function() { }

AbstractMap.prototype.setZoom 		= function(zoom) { }
AbstractMap.prototype.setCenter 	= function(ll) { }
AbstractMap.prototype.pixToLL 		= function (point) { }
AbstractMap.prototype.llToPix 		= function (latLng) { }
AbstractMap.prototype.fromLatLngToDivPixel = function (latLng) { return null; }
AbstractMap.prototype.fromDivPixelToLatLng = function (xy) { return null; }
AbstractMap.prototype.useWheel		= function () { }
AbstractMap.prototype.getZoom		= function () { return 0; }
AbstractMap.prototype.setZoom		= function (zoom) { }
AbstractMap.prototype.placeCustomMarker	= function (lat, lon, url, w, h) { }
AbstractMap.prototype.placeHtmlMarker	= function (lat, lon, args) { }
AbstractMap.prototype.placeCustomHtmlMarker	= function (lat, lon, args) { }
AbstractMap.prototype.getCenterLat	= function () { return 0; }
AbstractMap.prototype.getCenterLon	= function () { return 0; }
AbstractMap.prototype.getBoundsN	= function () { return 0; }
AbstractMap.prototype.getBoundsS	= function () { return 0; }
AbstractMap.prototype.getBoundsW	= function () { return 0; }
AbstractMap.prototype.getBoundsE	= function () { return 0; }
AbstractMap.prototype.newLLPoint	= function (lat, lng) { return null; }
AbstractMap.prototype.newXYPoint	= function (x, y) { return null; }
AbstractMap.prototype.newGroundOverlay	= function (url, sw, ne, opacity, clickable) { return null; }
AbstractMap.prototype.newPolygon	= function (options) { return null; }
AbstractMap.prototype.newPolyline	= function (options) { return null; }
AbstractMap.prototype.newMarker		= function (options, clickCallback, mouseOverCallback, mouseOutCallback) { return null; }
AbstractMap.prototype.newSize		= function (w, h) { return null; }
AbstractMap.prototype.newInfoWindow	= function (options, reference) { return null; }
AbstractMap.prototype.getDiv = function () { return null; } //map's div

// --------

// Concrete implementation
function GmapMap(element, options) {
	AbstractMap.call(this, element, new google.maps.Map(element, options));
}

GmapMap.prototype = Object.create(AbstractMap.prototype);
GmapMap.prototype.constructor = GmapMap;

GmapMap.prototype.init = function() {
	var self = this;
	google.maps.event.addListener(this.map, 'zoom_changed', function(){self.onZoom();});
	google.maps.event.addListener(this.map, 'dragstart', 	function(){self.onDragStart();});
	google.maps.event.addListener(this.map, 'drag', 		function(){self.onDrag();});
	google.maps.event.addListener(this.map, 'dragend', 		function(){self.onDragEnd();});
	google.maps.event.addListener(this.map, 'idle', 		function(){self.onIdle();});
	this.element.addEventListener('touchstart', function(){self.onTouchStart();}, false);
	this.element.addEventListener('touchmove', 	function(){self.onTouchMove();}, false);
	this.element.addEventListener('touchend', 	function(){self.onTouchEnd();}, false);

	function ProjectionOverlay(map) { this.setMap(map); }
	ProjectionOverlay.prototype = new google.maps.OverlayView();
	ProjectionOverlay.prototype.onAdd = function() { }
	ProjectionOverlay.prototype.onRemove = function() { }
	ProjectionOverlay.prototype.draw = function() { }
	this.overlay = new ProjectionOverlay(this.map);

	setTimeout(this.finalize.bind(this), 50);
}

GmapMap.prototype.finalize 		= function() {
	this.projection = this.overlay.getProjection();
	if(!this.projection) {
		setTimeout(this.finalize.bind(this), 50);
	} else {
		this.onReady();
	}
}
GmapMap.prototype.setZoom 		= function(zoom) {
	this.map.setZoom(zoom);
}
GmapMap.prototype.getZoom		= function () {
	return this.map.getZoom();
}

GmapMap.prototype.setCenter 	= function(ll) {
	this.map.setCenter(ll);
}

GmapMap.prototype.pixToLL 		= function (point) {
	var ne = this.map.getBounds().getNorthEast();
	var sw = this.map.getBounds().getSouthWest();
	var projection = this.map.getProjection();
	var topRight = projection.fromLatLngToPoint(ne);
	var bottomLeft = projection.fromLatLngToPoint(sw);
	var scale = 1 << this.map.getZoom();

	var newLatlng = projection.fromPointToLatLng(new google.maps.Point(point.x / scale + bottomLeft.x, point.y / scale + topRight.y));
	return newLatlng;
}

GmapMap.prototype.llToPix 		= function (latLng) {
	var topRight = this.map.getProjection().fromLatLngToPoint(this.map.getBounds().getNorthEast());
	var bottomLeft = this.map.getProjection().fromLatLngToPoint(this.map.getBounds().getSouthWest());
	var scale = Math.pow(2, this.map.getZoom());
	var worldPoint = this.map.getProjection().fromLatLngToPoint(latLng);
	return new google.maps.Point((worldPoint.x-bottomLeft.x)*scale, (worldPoint.y-topRight.y)*scale);
}

GmapMap.prototype.useWheel		= function () {
	this.map.setOptions({scrollwheel:true});
}

GmapMap.prototype.getZoom		= function () {
	return this.map.getZoom();
}

GmapMap.prototype.placeCustomMarker	= function (lat, lon, url, w, h) {
	var centerPoint = new google.maps.LatLng(lat, lon);
	var image = {
		url: url,
		size: new google.maps.Size(w, h)
	};
	var centerPointIcon = new google.maps.Marker({
		map: map,
		position: centerPoint,
		icon: image
	 });
}

GmapMap.prototype.getCenterLat	= function () { return this.map.getCenter().lat(); }
GmapMap.prototype.getCenterLon	= function () { return this.map.getCenter().lng(); }
GmapMap.prototype.getBoundsN	= function () { return this.map.getBounds().getNorthEast().lat(); }
GmapMap.prototype.getBoundsS	= function () { return this.map.getBounds().getSouthWest().lat(); }
GmapMap.prototype.getBoundsW	= function () { return this.map.getBounds().getSouthWest().lng(); }
GmapMap.prototype.getBoundsE	= function () { return this.map.getBounds().getNorthEast().lng(); }
GmapMap.prototype.newXYPoint	= function (x, y) { return new google.maps.Point(x, y); }
GmapMap.prototype.newLLPoint	= function (lat, lng) { return new google.maps.LatLng(lat, lng); }
GmapMap.prototype.newGroundOverlay	= function (url, sw, ne, opacity, clickable) {
	var overlay = new google.maps.GroundOverlay(url, new google.maps.LatLngBounds(sw, ne), {clickable: clickable});
	overlay.setOpacity(opacity);
	overlay.setMap(this.map);
	return overlay;
}
GmapMap.prototype.placeCustomHtmlMarker	= function (x, y, args) {
	return new CustomMarkerGmap({x: x, y: y}, this.map, args);
}
GmapMap.prototype.newPolygon	= function (options) {
	var p = new google.maps.Polygon(options);
	p.setMap(this.map);
	return p;
}
GmapMap.prototype.newPolyline	= function (options) {
	var p = new google.maps.Polyline(options);
	p.setMap(this.map);
	return p;
}
GmapMap.prototype.fromLatLngToDivPixel = function (latLng) {
	return this.projection.fromLatLngToDivPixel(latLng);
}
GmapMap.prototype.fromDivPixelToLatLng = function (xy) {
	return this.projection.fromDivPixelToLatLng(xy);
}
GmapMap.prototype.newSize		= function (w, h) {
	return new google.maps.Size(w, h);
}
GmapMap.prototype.newMarker		= function (options, clickCallback, mouseOverCallback, mouseOutCallback) {
	var m = new google.maps.Marker(options);
	m.setMap(this.map);
	clickCallback && 		m.addListener('click', 		clickCallback);
	mouseOverCallback && 	m.addListener('mouseover', 	mouseOverCallback);
	mouseOutCallback && 	m.addListener('mouseout', 	mouseOutCallback);
	return m;
}
GmapMap.prototype.newInfoWindow	= function (options, reference) {
	var infowindow = new google.maps.InfoWindow(options);
	infowindow.open(this.map, reference);
	return infowindow;
}
GmapMap.prototype.getDiv = function () { return this.map.getDiv(); }

GmapMap.prototype.placeHtmlMarker	= function (x, y, args) {
	return new HTMLMarker(x, y, this.map, args);
}

//@ sourceURL=js/map/abstract_map.js
